# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import vsm_settings as set_views
from vsm.openstack.common.db import exception as db_exc
from vsm import db
from vsm import utils
from vsm import exception
from vsm import scheduler

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_setting(elem, detailed=False):
    elem.set('id')
    elem.set('name')
    elem.set('value')
    elem.set('default_value')

    if detailed:
        pass

setting_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class SettingTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('setting', selector='setting')
        make_setting(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=setting_nsmap)

class SettingsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('settings')
        elem = xmlutil.SubTemplateElement(root, 'setting', selector='settings')
        make_setting(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=setting_nsmap)

class Controller(wsgi.Controller):
    """The Monitor API controller for the OpenStack API."""
    _view_builder_class = set_views.ViewBuilder

    def __init__(self, ext_mgr):
        super(Controller, self).__init__()
        self.scheduler_api = scheduler.API()

    @wsgi.serializers(xml=SettingsTemplate)
    @wsgi.response(202)
    def get_by_name(self, req):
        """ Get one vsm setting by name
        """
        context = req.environ['vsm.context']
        search_opts = {}
        search_opts.update(req.GET)

        LOG.debug('search options %s' % search_opts)
        vsm_name = search_opts.pop('name', None)
        if not vsm_name:
            raise exc.HTTPBadRequest(explanation=_('Invalid request: vsm name is required.'))

        try:
            utils.check_string_length(vsm_name, 'name', min_length=1, max_length=255)
            setting = db.vsm_settings_get_by_name(context, vsm_name)
        except db_exc.DBError as e:
            raise exc.HTTPServerError(explanation=e.message)
        except exception.InvalidInput as e:
            raise exc.HTTPBadRequest(explanation=e.message)

        if not setting:
            raise exc.HTTPNotFound(explanation=_('The vsm setting(%s) does not exists.' % vsm_name))

        return self._view_builder.basic(req, setting)

    @wsgi.serializers(xml=SettingsTemplate)
    def index(self, req):
        """Get a vsm setting list."""
        context = req.environ['vsm.context']

        settings = db.vsm_settings_get_all(context)
        LOG.info('vsm/api/v1/vsm_settings.py settings:%s' % settings)

        return self._view_builder.index(req, settings)

    @wsgi.serializers(xml=SettingsTemplate)
    def detail(self, req):
        """Get a detailed vsm setting list."""
        context = req.environ['vsm.context']

        settings = db.vsm_settings_get_all(context)
        LOG.info('vsm/api/v1/vsm_settings.py settings:%s' % settings)

        return self._view_builder.detail(req, settings)

    @wsgi.serializers(xml=SettingsTemplate)
    @wsgi.response(202)
    def create(self, req, body=None):
        """Create or update the vsm setting.
        {
         'setting': {
                        'name' : 'xxx',
                        'value': 'yyy'
                    }

        }
        """
        if not self.is_valid_body(body, 'setting'):
            raise exc.HTTPBadRequest(explanation=_('Invalid request body.'))

        context = req.environ['vsm.context']
        setting_dict = body['setting']

        self._validate_body(setting_dict)
        instance = self._create_setting(context, setting_dict)
        return self._view_builder.basic(req, instance)

    def _create_setting(self, context, setting_dict):

        try:
            if setting_dict.get('name') in ['cpu_diamond_collect_interval','ceph_diamond_collect_interval']:
                 self.scheduler_api.reconfig_diamond(context, setting_dict)
            return db.vsm_settings_update_or_create(context, setting_dict)

        except db_exc.DBError as e:
            raise exc.HTTPServerError(explanation=e.message)

    def _validate_body(self, setting_dict):
        if not isinstance(setting_dict, dict):
            raise exc.HTTPBadRequest(explanation=_('Invalid request body.'))

        if not 'name' in setting_dict.keys():
            LOG.debug('dict keys %s %s' % (setting_dict, setting_dict.keys()))
            msg = _("vsm setting name is not defined in vsm setting request.")
            raise exc.HTTPBadRequest(explanation=msg)

        name = setting_dict.get('name')
        value = setting_dict.get('value')
        default_value = setting_dict.get('default_value')

        try:
            if name:
                utils.check_string_length(name, 'name', min_length=1, max_length=255)
            else:
                msg = _('key name cannot be null or empty.')
                raise exc.HTTPBadRequest(explanation=msg)
            if value:
                utils.check_string_length(value, 'value', max_length=255)
            if default_value:
                utils.check_string_length(default_value, 'default value', max_length=255)
        except exception.InvalidInput as e:
            raise exc.HTTPBadRequest(explanation=e.message)

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Worker for Ceph..
"""

#import psutil
from vsm import utils
from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)

class CephWorker(object):
    """Worker for Ceph."""
    def __init__(self):
        pass




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging
import json

LOG = logging.getLogger(__name__)
import time

class ViewBuilder(common.ViewBuilder):
    _collection_name = "summary"

    def basic(self, summary, sum_type):
        dict_root = sum_type + '-summary'
        if not summary:
            return {
                dict_root: None
            }

        if (not summary.get('summary_data')) | (summary.get('summary_data') == 'null'):
            return {
                dict_root: None
            }

        try:
            updated_at = summary.get('updated_at').strftime("%Y-%m-%d %H:%M:%S")
        except:
            updated_at = ""

        sum_data = json.loads(summary['summary_data'])

        LOG.debug('summary data: %s' % sum_data)
        if sum_type == 'osd':
            sum_data = sum_data.get('osdmap')
            ret = {
                dict_root: {
                    'epoch': sum_data.get('epoch', 0),
                    'num_osds': sum_data.get('num_osds', 0),
                    'num_up_osds': sum_data.get('num_up_osds', 0),
                    'num_in_osds': sum_data.get('num_in_osds', 0),
                    'full': sum_data.get('full', None),
                    'nearfull': sum_data.get('nearfull', None),
                    'updated_at': updated_at,
                }
            }
            LOG.debug('return view %s' % ret)
            return ret
        elif sum_type == 'monitor':
            ret = {
                dict_root: {
                    'monmap_epoch': sum_data.get('monmap_epoch'),
                    'monitors': sum_data.get('monitors'),
                    'election_epoch': sum_data.get('election_epoch'),
                    'quorum': sum_data.get('quorum'),
                    'overall_status': sum_data.get('overall_status'),
                    'updated_at': updated_at,
                    'quorum_leader_name': sum_data.get('quorum_leader_name'),
                    'quorum_leader_rank': sum_data.get('quorum_leader_rank')
                }
            }
            LOG.debug('return view %s' % ret)
            return ret
        elif sum_type == "mds":
            ret = {
                dict_root: {
                    'epoch': sum_data.get('epoch'),
                    'num_up_mdses': sum_data.get('up'),
                    'num_in_mdses': sum_data.get('in'),
                    'num_max_mdses': sum_data.get('max'),
                    'num_failed_mdses': sum_data.get('failed'),
                    'num_stopped_mdses': sum_data.get('stopped'),
                    'metadata_pool': sum_data.get('metadata_pool'),
                    'data_pools': sum_data.get('data_pools'),
                    'updated_at': updated_at,
                }
            }
            LOG.debug('return view %s' % ret)
            return ret

        elif sum_type == "placement_group":
            ret = {
                dict_root: {
                    'version': sum_data.get('version'),
                    'num_pgs': sum_data.get('num_pgs'),
                    'pgs_by_state': sum_data.get('pgs_by_state'),
                    'data_bytes': sum_data.get('data_bytes', 0),
                    'bytes_used': sum_data.get('bytes_used', 0),
                    'bytes_avail': sum_data.get('bytes_avail', 0),
                    'bytes_total': sum_data.get('bytes_total', 0),
                    'degraded_objects': sum_data.get('degraded_objects', 0),
                    'degraded_total': sum_data.get('degraded_total', 0),
                    'degraded_ratio': sum_data.get('degraded_ratio', 0),
                    'unfound_objects': sum_data.get('unfound_objects', 0),
                    'unfound_total': sum_data.get('unfound_total', 0),
                    'unfound_ratio': sum_data.get('unfound_ratio', 0),
                    'read_bytes_sec': sum_data.get('read_bytes_sec', 0),
                    'write_bytes_sec': sum_data.get('write_bytes_sec', 0),
                    'op_per_sec': sum_data.get('op_per_sec', 0),
                    'updated_at': updated_at,
                }
            }
            LOG.debug('return view %s' % ret)
            return ret

        elif sum_type == "cluster":
            ret = {
                dict_root: {
                    'cluster': sum_data.get('cluster'),
                    'status': sum_data.get('status'),
                    'detail': sum_data.get('detail'),
                    'health_list': sum_data.get('health_list'),
                    'updated_at': updated_at,
                }
            }
            LOG.debug('return view %s' % ret)
            return ret

        elif sum_type == "vsm":
            ret = {
                dict_root: {
                    'uptime': sum_data.get('uptime'),
                    'ceph_version': sum_data.get('ceph_version'),
                    'created_at': sum_data.get('created_at'),
                    'is_ceph_active': sum_data.get('is_ceph_active'),
                    'updated_at': updated_at,
                }
            }
            LOG.debug('return view %s' % ret)
            return ret

    #def index(self, summary, sum_type):
    #    """Show summary without many details."""
    #    return self._list_view(self.basic, summary, sum_type)
    #
    #def _list_view(self, func, summary):
    #    """Provide a view for summary."""
    #    agents_list = [func(summary)["summary"] for agents in agentss]
    #    agentss_dict = dict(agentss=agents_list)
    #    return agentss_dict



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging
from vsm import db
import time

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "osds"

    def __init__(self):
        self._host = ''

    def basic(self, request, osd):
        LOG.info("osds api view %s " % osd)
        return {
            "osd": {
                "id":osd['id'],
                "osd_name": osd['osd_name'],
                "state": osd['state'],
                "operation_status": osd['operation_status'],
                "weight": osd['weight'],
                "device_id": osd['device_id'],
                "service_id": osd['service_id'],
                "updated_at": osd['updated_at'],
            }
        }

    def _detail(self, request, osd):
        LOG.info("osds api detail view %s " % osd)
        _osd = {
            "osd": {
                "id": osd['id'],
                "osd_name": osd['osd_name'],
                "state": osd.get('state', ''),
                "operation_status": osd.get('operation_status', ''),
                "weight": osd.get('weight', 0.0),
                #? "crush_weight": osd['crush_weight'],
                "device_id": osd['device_id'],
                "service_id": osd['service_id'],
                "service": osd['service'],
                "storage_group": osd['storage_group'],
                "zone": self._get_zone(request, osd),
                "device": osd['device'],
            }
        }
        try:
            _osd['osd']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(osd['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        except:
            _osd['osd']['updated_at'] = ""

        return _osd

    def _get_zone(self, req, osd):
        """get zone name from server name"""
        zone = ''
        context = req.environ["vsm.context"]
        host = self._get_server(osd)
        if not host:
            return zone

        node = db.init_node_get_by_host(context=context, host=host)
        if node and node.zone:
            zone = node.zone.name

        return zone

    def _get_server(self, osd):
        if self._host:
            return self._host

        if osd and osd.get('service'):
            server = osd['service']['host']

        return server

    def detail(self, request, osds):
        return self._list_view(self._detail, request, osds)

    def index(self, request, osds):
        """Show a list of osds without many details."""
        return self._list_view(self.basic, request, osds)

    def _list_view(self, func, request, osds):
        """Provide a view for a list of osds."""
        osd_list = [func(request, osd)["osd"] for osd in osds]
        osds_dict = dict(osds=osd_list)
        return osds_dict




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 OpenStack Foundation.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Openstack logging handler.

This module adds to logging functionality by adding the option to specify
a context object when calling the various log methods.  If the context object
is not specified, default formatting is used. Additionally, an instance uuid
may be passed as part of the log message, which is intended to make it easier
for admins to find messages related to a specific instance.

It also allows setting of formatting information through conf.

"""

import cStringIO
import inspect
import itertools
import logging
import logging.config
import logging.handlers
import os
import stat
import sys
import traceback

from oslo.config import cfg

from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import jsonutils
from vsm.openstack.common import local
from vsm.openstack.common import notifier

_DEFAULT_LOG_FORMAT = "%(asctime)s %(levelname)8s [%(name)s] %(message)s"
_DEFAULT_LOG_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

common_cli_opts = [
    cfg.BoolOpt('debug',
                short='d',
                default=False,
                help='Print debugging output (set logging level to '
                     'DEBUG instead of default WARNING level).'),
    cfg.BoolOpt('verbose',
                short='v',
                default=True,
                help='Print more verbose output (set logging level to '
                     'INFO instead of default WARNING level).'),
]

logging_cli_opts = [
    cfg.StrOpt('log-config',
               metavar='PATH',
               help='If this option is specified, the logging configuration '
                    'file specified is used and overrides any other logging '
                    'options specified. Please see the Python logging module '
                    'documentation for details on logging configuration '
                    'files.'),
    cfg.StrOpt('log-format',
               default=_DEFAULT_LOG_FORMAT,
               metavar='FORMAT',
               help='A logging.Formatter log message format string which may '
                    'use any of the available logging.LogRecord attributes. '
                    'Default: %(default)s'),
    cfg.StrOpt('log-date-format',
               default=_DEFAULT_LOG_DATE_FORMAT,
               metavar='DATE_FORMAT',
               help='Format string for %%(asctime)s in log records. '
                    'Default: %(default)s'),
    cfg.StrOpt('log-file',
               metavar='PATH',
               deprecated_name='logfile',
               help='(Optional) Name of log file to output to. '
                    'If not set, logging will go to stdout.'),
    cfg.StrOpt('log-dir',
               deprecated_name='logdir',
               help='(Optional) The directory to keep log files in '
                    '(will be prepended to --log-file)'),
    cfg.BoolOpt('use-syslog',
                default=False,
                help='Use syslog for logging.'),
    cfg.StrOpt('syslog-log-facility',
               default='LOG_USER',
               help='syslog facility to receive log lines')
]

generic_log_opts = [
    cfg.BoolOpt('use_stderr',
                default=True,
                help='Log output to standard error'),
    cfg.StrOpt('logfile_mode',
               default='0644',
               help='Default file mode used when creating log files'),
]

log_opts = [
    cfg.StrOpt('logging_context_format_string',
               default='%(asctime)s.%(msecs)03d %(levelname)s %(name)s '
                       '[%(request_id)s %(user)s %(tenant)s] %(instance)s'
                       '%(message)s',
               help='format string to use for log messages with context'),
    cfg.StrOpt('logging_default_format_string',
               default='%(asctime)s.%(msecs)03d %(process)d %(levelname)s '
                       '%(name)s [-] %(instance)s%(message)s',
               help='format string to use for log messages without context'),
    cfg.StrOpt('logging_debug_format_suffix',
               default='%(funcName)s %(pathname)s:%(lineno)d',
               help='data to append to log format when level is DEBUG'),
    cfg.StrOpt('logging_exception_prefix',
               default='%(asctime)s.%(msecs)03d %(process)d TRACE %(name)s '
               '%(instance)s',
               help='prefix each line of exception output with this format'),
    cfg.ListOpt('default_log_levels',
                default=[
                    'amqplib=WARN',
                    'sqlalchemy=WARN',
                    'boto=WARN',
                    'suds=INFO',
                    'keystone=INFO',
                    'eventlet.wsgi.server=WARN'
                ],
                help='list of logger=LEVEL pairs'),
    cfg.BoolOpt('publish_errors',
                default=False,
                help='publish error events'),
    cfg.BoolOpt('fatal_deprecations',
                default=False,
                help='make deprecations fatal'),

    # NOTE(mikal): there are two options here because sometimes we are handed
    # a full instance (and could include more information), and other times we
    # are just handed a UUID for the instance.
    cfg.StrOpt('instance_format',
               default='[instance: %(uuid)s] ',
               help='If an instance is passed with the log message, format '
                    'it like this'),
    cfg.StrOpt('instance_uuid_format',
               default='[instance: %(uuid)s] ',
               help='If an instance UUID is passed with the log message, '
                    'format it like this'),
]

CONF = cfg.CONF
CONF.register_cli_opts(common_cli_opts)
CONF.register_cli_opts(logging_cli_opts)
CONF.register_opts(generic_log_opts)
CONF.register_opts(log_opts)

# our new audit level
# NOTE(jkoelker) Since we synthesized an audit level, make the logging
#                module aware of it so it acts like other levels.
logging.AUDIT = logging.INFO + 1
logging.addLevelName(logging.AUDIT, 'AUDIT')

try:
    NullHandler = logging.NullHandler
except AttributeError:  # NOTE(jkoelker) NullHandler added in Python 2.7
    class NullHandler(logging.Handler):
        def handle(self, record):
            pass

        def emit(self, record):
            pass

        def createLock(self):
            self.lock = None

def _dictify_context(context):
    if context is None:
        return None
    if not isinstance(context, dict) and getattr(context, 'to_dict', None):
        context = context.to_dict()
    return context

def _get_binary_name():
    return os.path.basename(inspect.stack()[-1][1])

def _get_log_file_path(binary=None):
    logfile = CONF.log_file
    logdir = CONF.log_dir

    if logfile and not logdir:
        return logfile

    if logfile and logdir:
        return os.path.join(logdir, logfile)

    if logdir:
        binary = binary or _get_binary_name()
        return '%s.log' % (os.path.join(logdir, binary),)

class ContextAdapter(logging.LoggerAdapter):
    warn = logging.LoggerAdapter.warning

    def __init__(self, logger, project_name, version_string):
        self.logger = logger
        self.project = project_name
        self.version = version_string

    def audit(self, msg, *args, **kwargs):
        self.log(logging.AUDIT, msg, *args, **kwargs)

    def deprecated(self, msg, *args, **kwargs):
        stdmsg = _("Deprecated: %s") % msg
        if CONF.fatal_deprecations:
            self.critical(stdmsg, *args, **kwargs)
            raise DeprecatedConfig(msg=stdmsg)
        else:
            self.warn(stdmsg, *args, **kwargs)

    def process(self, msg, kwargs):
        if 'extra' not in kwargs:
            kwargs['extra'] = {}
        extra = kwargs['extra']

        context = kwargs.pop('context', None)
        if not context:
            context = getattr(local.store, 'context', None)
        if context:
            extra.update(_dictify_context(context))

        instance = kwargs.pop('instance', None)
        instance_extra = ''
        if instance:
            instance_extra = CONF.instance_format % instance
        else:
            instance_uuid = kwargs.pop('instance_uuid', None)
            if instance_uuid:
                instance_extra = (CONF.instance_uuid_format
                                  % {'uuid': instance_uuid})
        extra.update({'instance': instance_extra})

        extra.update({"project": self.project})
        extra.update({"version": self.version})
        extra['extra'] = extra.copy()
        return msg, kwargs

class JSONFormatter(logging.Formatter):
    def __init__(self, fmt=None, datefmt=None):
        # NOTE(jkoelker) we ignore the fmt argument, but its still there
        #                since logging.config.fileConfig passes it.
        self.datefmt = datefmt

    def formatException(self, ei, strip_newlines=True):
        lines = traceback.format_exception(*ei)
        if strip_newlines:
            lines = [itertools.ifilter(
                lambda x: x,
                line.rstrip().splitlines()) for line in lines]
            lines = list(itertools.chain(*lines))
        return lines

    def format(self, record):
        message = {'message': record.getMessage(),
                   'asctime': self.formatTime(record, self.datefmt),
                   'name': record.name,
                   'msg': record.msg,
                   'args': record.args,
                   'levelname': record.levelname,
                   'levelno': record.levelno,
                   'pathname': record.pathname,
                   'filename': record.filename,
                   'module': record.module,
                   'lineno': record.lineno,
                   'funcname': record.funcName,
                   'created': record.created,
                   'msecs': record.msecs,
                   'relative_created': record.relativeCreated,
                   'thread': record.thread,
                   'thread_name': record.threadName,
                   'process_name': record.processName,
                   'process': record.process,
                   'traceback': None}

        if hasattr(record, 'extra'):
            message['extra'] = record.extra

        if record.exc_info:
            message['traceback'] = self.formatException(record.exc_info)

        return jsonutils.dumps(message)

class PublishErrorsHandler(logging.Handler):
    def emit(self, record):
        if ('vsm.openstack.common.notifier.log_notifier' in
                CONF.notification_driver):
            return
        notifier.api.notify(None, 'error.publisher',
                            'error_notification',
                            notifier.api.ERROR,
                            dict(error=record.msg))

def _create_logging_excepthook(product_name):
    def logging_excepthook(type, value, tb):
        extra = {}
        if CONF.verbose:
            extra['exc_info'] = (type, value, tb)
        getLogger(product_name).critical(str(value), **extra)
    return logging_excepthook

def setup(product_name):
    """Setup logging."""
    if CONF.log_config:
        logging.config.fileConfig(CONF.log_config)
    else:
        _setup_logging_from_conf()
    sys.excepthook = _create_logging_excepthook(product_name)

def set_defaults(logging_context_format_string):
    cfg.set_defaults(log_opts,
                     logging_context_format_string=
                     logging_context_format_string)

def _find_facility_from_conf():
    facility_names = logging.handlers.SysLogHandler.facility_names
    facility = getattr(logging.handlers.SysLogHandler,
                       CONF.syslog_log_facility,
                       None)

    if facility is None and CONF.syslog_log_facility in facility_names:
        facility = facility_names.get(CONF.syslog_log_facility)

    if facility is None:
        valid_facilities = facility_names.keys()
        consts = ['LOG_AUTH', 'LOG_AUTHPRIV', 'LOG_CRON', 'LOG_DAEMON',
                  'LOG_FTP', 'LOG_KERN', 'LOG_LPR', 'LOG_MAIL', 'LOG_NEWS',
                  'LOG_AUTH', 'LOG_SYSLOG', 'LOG_USER', 'LOG_UUCP',
                  'LOG_LOCAL0', 'LOG_LOCAL1', 'LOG_LOCAL2', 'LOG_LOCAL3',
                  'LOG_LOCAL4', 'LOG_LOCAL5', 'LOG_LOCAL6', 'LOG_LOCAL7']
        valid_facilities.extend(consts)
        raise TypeError(_('syslog facility must be one of: %s') %
                        ', '.join("'%s'" % fac
                                  for fac in valid_facilities))

    return facility

def _setup_logging_from_conf():
    log_root = getLogger(None).logger
    for handler in log_root.handlers:
        log_root.removeHandler(handler)

    if CONF.use_syslog:
        facility = _find_facility_from_conf()
        syslog = logging.handlers.SysLogHandler(address='/dev/log',
                                                facility=facility)
        log_root.addHandler(syslog)

    logpath = _get_log_file_path()
    if logpath:
        filelog = logging.handlers.WatchedFileHandler(logpath)
        #filelog = logging.handlers.TimedRotatingFileHandler(logpath,
        #                                        when='M',
        #                                        interval=5,
        #                                        backupCount=5)
        log_root.addHandler(filelog)

        mode = int(CONF.logfile_mode, 8)
        st = os.stat(logpath)
        if st.st_mode != (stat.S_IFREG | mode):
            os.chmod(logpath, mode)

    if CONF.use_stderr:
        streamlog = ColorHandler()
        log_root.addHandler(streamlog)

    elif not CONF.log_file:
        # pass sys.stdout as a positional argument
        # python2.6 calls the argument strm, in 2.7 it's stream
        streamlog = logging.StreamHandler(sys.stdout)
        log_root.addHandler(streamlog)

    if CONF.publish_errors:
        log_root.addHandler(PublishErrorsHandler(logging.ERROR))

    for handler in log_root.handlers:
        datefmt = CONF.log_date_format
        if CONF.log_format:
            handler.setFormatter(logging.Formatter(fmt=CONF.log_format,
                                                   datefmt=datefmt))
        else:
            handler.setFormatter(LegacyFormatter(datefmt=datefmt))

    if CONF.debug:
        log_root.setLevel(logging.DEBUG)
    elif CONF.verbose:
        log_root.setLevel(logging.INFO)
    else:
        log_root.setLevel(logging.WARNING)

    level = logging.NOTSET
    for pair in CONF.default_log_levels:
        mod, _sep, level_name = pair.partition('=')
        level = logging.getLevelName(level_name)
        logger = logging.getLogger(mod)
        logger.setLevel(level)
        for handler in log_root.handlers:
            logger.addHandler(handler)

_loggers = {}

def getLogger(name='unknown', version='unknown'):
    if name not in _loggers:
        _loggers[name] = ContextAdapter(logging.getLogger(name),
                                        name,
                                        version)
    return _loggers[name]

class WritableLogger(object):
    """A thin wrapper that responds to `write` and logs."""

    def __init__(self, logger, level=logging.INFO):
        self.logger = logger
        self.level = level

    def write(self, msg):
        self.logger.log(self.level, msg)

class LegacyFormatter(logging.Formatter):
    """A context.RequestContext aware formatter configured through flags.

    The flags used to set format strings are: logging_context_format_string
    and logging_default_format_string.  You can also specify
    logging_debug_format_suffix to append extra formatting if the log level is
    debug.

    For information about what variables are available for the formatter see:
    http://docs.python.org/library/logging.html#formatter

    """

    def format(self, record):
        """Uses contextstring if request_id is set, otherwise default."""
        # NOTE(sdague): default the fancier formating params
        # to an empty string so we don't throw an exception if
        # they get used
        for key in ('instance', 'color'):
            if key not in record.__dict__:
                record.__dict__[key] = ''

        if record.__dict__.get('request_id', None):
            self._fmt = CONF.logging_context_format_string
        else:
            self._fmt = CONF.logging_default_format_string

        if (record.levelno == logging.DEBUG and
                CONF.logging_debug_format_suffix):
            self._fmt += " " + CONF.logging_debug_format_suffix

        # Cache this on the record, Logger will respect our formated copy
        if record.exc_info:
            record.exc_text = self.formatException(record.exc_info, record)
        return logging.Formatter.format(self, record)

    def formatException(self, exc_info, record=None):
        """Format exception output with CONF.logging_exception_prefix."""
        if not record:
            return logging.Formatter.formatException(self, exc_info)

        stringbuffer = cStringIO.StringIO()
        traceback.print_exception(exc_info[0], exc_info[1], exc_info[2],
                                  None, stringbuffer)
        lines = stringbuffer.getvalue().split('\n')
        stringbuffer.close()

        if CONF.logging_exception_prefix.find('%(asctime)') != -1:
            record.asctime = self.formatTime(record, self.datefmt)

        formatted_lines = []
        for line in lines:
            pl = CONF.logging_exception_prefix % record.__dict__
            fl = '%s%s' % (pl, line)
            formatted_lines.append(fl)
        return '\n'.join(formatted_lines)

class ColorHandler(logging.StreamHandler):
    LEVEL_COLORS = {
        logging.DEBUG: '\033[00;32m',  # GREEN
        logging.INFO: '\033[00;36m',  # CYAN
        logging.AUDIT: '\033[01;36m',  # BOLD CYAN
        logging.WARN: '\033[01;33m',  # BOLD YELLOW
        logging.ERROR: '\033[01;31m',  # BOLD RED
        logging.CRITICAL: '\033[01;31m',  # BOLD RED
    }

    def format(self, record):
        record.color = self.LEVEL_COLORS[record.levelno]
        return logging.StreamHandler.format(self, record)

class DeprecatedConfig(Exception):
    message = _("Fatal call to deprecated config: %(msg)s")

    def __init__(self, msg):
        super(Exception, self).__init__(self.message % dict(msg=msg))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "poolusages"

    def basic(self, poolusage):
        if not poolusage.get('id', ""):
            poolusage['id'] = ""
            return poolusage
        return {
            "poolusage": {
                "id": poolusage.get("id", 0),
                "pool_id": poolusage.get("pool_id", ""),
                "vsmapp_id": poolusage.get("vsmapp_id", ""),
                "cinder_volume_host": poolusage.get("cinder_volume_host", ""),
                "as_glance_store_pool": poolusage.get("as_glance_store_pool", ""),
                "attach_status": poolusage.get("attach_status", ""),
                "attach_at": poolusage.get("attach_at", "")
            }
        }

    def index(self, poolusages):
        """Show a list of poolusages without many details."""
        return self._list_view(self.basic, poolusages)

    def _list_view(self, func, poolusages):
        """Provide a view for a list of poolusages."""
        node_list = [func(poolusage)["poolusage"] for poolusage in poolusages]
        nodes_dict = dict(poolusages=node_list)
        return nodes_dict



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Contrib contains extensions that are shipped with vsm.

It can't be called 'extensions' because that causes namespacing problems.

"""

from vsm.api import extensions
from vsm import flags
from vsm.openstack.common import log as logging

FLAGS = flags.FLAGS
LOG = logging.getLogger(__name__)

def standard_extensions(ext_mgr):
    extensions.load_standard_extensions(ext_mgr, LOG, __path__, __package__)

def select_extensions(ext_mgr):
    extensions.load_standard_extensions(ext_mgr, LOG, __path__, __package__,
                                        FLAGS.vsmapi_storage_ext_list)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "servers"

    def basic(self, server):
        if not server['type']:
            server['type'] = "";
        return {
            "server": {
                "id": server["id"], 
                "host": server["host"], 
                "primary_public_ip": server["primary_public_ip"], 
                "secondary_public_ip": server["secondary_public_ip"], 
                "cluster_ip": server["cluster_ip"],
                "raw_ip":"192.168.1.3,192.168.2.3,192.168.3.3",
                "zone_id": server["zone_id"],
                "ceph_ver": server["ceph_ver"],
                "service_id": server["service_id"],
                "osds": server['data_drives_number'],
                "type": server['type'],
                "status": server['status']
            }
        }

    def index(self, servers):
        """Show a list of servers without many details."""
        return self._list_view(self.basic, servers)

    def _list_view(self, func, servers):
        """Provide a view for a list of servers."""
        server_list = [func(server)["server"] for server in servers]
        servers_dict = dict(servers=server_list)
        return servers_dict

